@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2016-2022 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2016-2023 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -38,17 +38,17 @@
 #include "asc-image.h"
 
 struct {
-	gint	width;
-	gint	height;
+	gint width;
+	gint height;
 } target_screenshot_sizes[] = {
-	{ 1248, 702 },
-	{ 752, 423 },
-	{ 624, 351 },
-	{ 224, 126 },
-	{ 0, 0 }
+	{1248, 702},
+	{ 752, 423},
+	{ 624, 351},
+	{ 224, 126},
+	{ 0,   0  }
 };
 
-static AscVideoInfo*
+static AscVideoInfo *
 asc_video_info_new (void)
 {
 	AscVideoInfo *vinfo;
@@ -67,14 +67,12 @@ asc_video_info_free (AscVideoInfo *vinfo)
 	g_free (vinfo);
 }
 
-
 static gsize
 asc_get_filesize (const gchar *filename)
 {
 	struct stat st;
 	if (lstat (filename, &st) == -1) {
-		g_debug ("Unable to determine size of file '%s': %s",
-			 filename, g_strerror (errno));
+		g_debug ("Unable to determine size of file '%s': %s", filename, g_strerror (errno));
 		return 0;
 	}
 	return st.st_size;
@@ -83,7 +81,7 @@ asc_get_filesize (const gchar *filename)
 /**
  * asc_extract_video_info: (skip):
  */
-AscVideoInfo*
+AscVideoInfo *
 asc_extract_video_info (AscResult *cres, AsComponent *cpt, const gchar *vid_fname)
 {
 	AscVideoInfo *vinfo = NULL;
@@ -96,15 +94,17 @@ asc_extract_video_info (AscResult *cres, AsComponent *cpt, const gchar *vid_fnam
 	g_autofree gchar *prev_codec_name = NULL;
 	gboolean audio_okay = FALSE;
 	g_autoptr(GError) error = NULL;
-	const gchar *ffprobe_argv[] = {
-		asc_globals_get_ffprobe_binary (),
-		"-v", "quiet",
-		"-show_entries", "stream=width,height,codec_name,codec_type",
-		"-show_entries", "format=format_name",
-		"-of", "default=noprint_wrappers=1",
-		vid_fname,
-		NULL
-	};
+	const gchar *ffprobe_argv[] = { asc_globals_get_ffprobe_binary (),
+					"-v",
+					"quiet",
+					"-show_entries",
+					"stream=width,height,codec_name,codec_type",
+					"-show_entries",
+					"format=format_name",
+					"-of",
+					"default=noprint_wrappers=1",
+					vid_fname,
+					NULL };
 
 	vinfo = asc_video_info_new ();
 	if (vid_fname == NULL)
@@ -114,7 +114,7 @@ asc_extract_video_info (AscResult *cres, AsComponent *cpt, const gchar *vid_fnam
 	vid_basename = g_path_get_basename (vid_fname);
 
 	ret = g_spawn_sync (NULL, /* working directory */
-			    (gchar**) ffprobe_argv,
+			    (gchar **) ffprobe_argv,
 			    NULL, /* envp */
 			    G_SPAWN_LEAVE_DESCRIPTORS_OPEN,
 			    NULL, /* child setup */
@@ -128,8 +128,10 @@ asc_extract_video_info (AscResult *cres, AsComponent *cpt, const gchar *vid_fnam
 		asc_result_add_hint (cres,
 				     cpt,
 				     "screenshot-video-check-failed",
-				     "fname", vid_basename,
-				     "msg", error->message,
+				     "fname",
+				     vid_basename,
+				     "msg",
+				     error->message,
 				     NULL);
 		return vinfo;
 	}
@@ -142,13 +144,18 @@ asc_extract_video_info (AscResult *cres, AsComponent *cpt, const gchar *vid_fnam
 			g_autofree gchar *dummy = ff_stderr;
 			ff_stderr = g_strconcat (ff_stderr, "\n", ff_stdout, NULL);
 		}
-		g_warning ("FFprobe on '%s' failed with error code %i: %s", vid_fname, exit_status, ff_stderr);
+		g_warning ("FFprobe on '%s' failed with error code %i: %s",
+			   vid_fname,
+			   exit_status,
+			   ff_stderr);
 		msg = g_strdup_printf ("Code %i, %s", exit_status, ff_stderr);
 		asc_result_add_hint (cres,
 				     cpt,
 				     "screenshot-video-check-failed",
-				     "fname", vid_basename,
-				     "msg", msg,
+				     "fname",
+				     vid_basename,
+				     "msg",
+				     msg,
 				     NULL);
 		return vinfo;
 	}
@@ -222,14 +229,17 @@ asc_extract_video_info (AscResult *cres, AsComponent *cpt, const gchar *vid_fnam
 		asc_result_add_hint (cres,
 				     cpt,
 				     "screenshot-video-has-audio",
-				     "fname", vid_basename,
+				     "fname",
+				     vid_basename,
 				     NULL);
 		if (g_strcmp0 (vinfo->audio_codec_name, "opus") != 0) {
 			asc_result_add_hint (cres,
 					     cpt,
 					     "screenshot-video-audio-codec-unsupported",
-					     "fname", vid_basename,
-					     "codec", vinfo->audio_codec_name,
+					     "fname",
+					     vid_basename,
+					     "codec",
+					     vinfo->audio_codec_name,
 					     NULL);
 			audio_okay = FALSE;
 		}
@@ -238,15 +248,18 @@ asc_extract_video_info (AscResult *cres, AsComponent *cpt, const gchar *vid_fnam
 	/* A video file may contain multiple streams, so this check isn't extensive, but it protects against 99% of cases where
 	 * people were using unsupported formats. */
 	vinfo->is_acceptable = (vinfo->container_kind != AS_VIDEO_CONTAINER_KIND_UNKNOWN) &&
-				(vinfo->codec_kind != AS_VIDEO_CODEC_KIND_UNKNOWN) && audio_okay;
+			       (vinfo->codec_kind != AS_VIDEO_CODEC_KIND_UNKNOWN) && audio_okay;
 
 	if (!vinfo->is_acceptable) {
 		asc_result_add_hint (cres,
 				     cpt,
 				     "screenshot-video-format-unsupported",
-				     "fname", vid_basename,
-				     "codec", vinfo->codec_name,
-				     "container", vinfo->format_name,
+				     "fname",
+				     vid_basename,
+				     "codec",
+				     vinfo->codec_name,
+				     "container",
+				     vinfo->format_name,
 				     NULL);
 	}
 
@@ -256,7 +269,7 @@ asc_extract_video_info (AscResult *cres, AsComponent *cpt, const gchar *vid_fnam
 /**
  * asc_process_screenshot_videos: (skip):
  */
-static AsScreenshot*
+static AsScreenshot *
 asc_process_screenshot_videos (AscResult *cres,
 			       AsComponent *cpt,
 			       AsScreenshot *scr,
@@ -305,10 +318,7 @@ asc_process_screenshot_videos (AscResult *cres,
 		fname_from_url = asc_filename_from_url (orig_vid_url);
 
 		if (g_strcmp0 (video_locale, "C") == 0)
-			scr_vid_name = g_strdup_printf ("vid%i-%i_%s",
-							scr_no,
-							i,
-							fname_from_url);
+			scr_vid_name = g_strdup_printf ("vid%i-%i_%s", scr_no, i, fname_from_url);
 		else
 			scr_vid_name = g_strdup_printf ("vid%i-%i_%s_%s",
 							scr_no,
@@ -320,11 +330,13 @@ asc_process_screenshot_videos (AscResult *cres,
 
 		if (!as_curl_download_to_filename (acurl, orig_vid_url, scr_vid_path, &error)) {
 			asc_result_add_hint (cres,
-						cpt,
-						"screenshot-download-error",
-						"url", orig_vid_url,
-						"error", error->message,
-						NULL);
+					     cpt,
+					     "screenshot-download-error",
+					     "url",
+					     orig_vid_url,
+					     "error",
+					     error->message,
+					     NULL);
 			return NULL;
 		}
 
@@ -338,9 +350,12 @@ asc_process_screenshot_videos (AscResult *cres,
 			asc_result_add_hint (cres,
 					     cpt,
 					     "screenshot-video-too-big",
-					     "fname", scr_vid_name,
-					     "max_size", max_vid_size_str,
-					     "size", vid_size_str,
+					     "fname",
+					     scr_vid_name,
+					     "max_size",
+					     max_vid_size_str,
+					     "size",
+					     vid_size_str,
 					     NULL);
 			g_unlink (scr_vid_path);
 			continue;
@@ -390,8 +405,7 @@ asc_process_screenshot_videos (AscResult *cres,
 
 	/* add the valid ones back */
 	for (guint i = 0; i < valid_vids->len; i++)
-		as_screenshot_add_video (scr,
-					 AS_VIDEO (g_ptr_array_index (valid_vids, i)));
+		as_screenshot_add_video (scr, AS_VIDEO (g_ptr_array_index (valid_vids, i)));
 
 	return g_object_ref (scr);
 }
@@ -423,6 +437,7 @@ asc_process_screenshot_images_lang (AscResult *cres,
 	gsize img_data_len;
 	guint source_scr_width;
 	guint source_scr_height;
+	guint source_scr_scale;
 	gboolean thumbnails_generated = FALSE;
 	g_autoptr(GError) error = NULL;
 
@@ -440,8 +455,10 @@ asc_process_screenshot_images_lang (AscResult *cres,
 		asc_result_add_hint (cres,
 				     cpt,
 				     "screenshot-download-error",
-				     "url", orig_img_url,
-				     "error", error->message,
+				     "url",
+				     orig_img_url,
+				     "error",
+				     error->message,
 				     NULL);
 		return FALSE;
 	}
@@ -449,11 +466,13 @@ asc_process_screenshot_images_lang (AscResult *cres,
 
 	gcid = asc_result_gcid_for_component (cres, cpt);
 	if (gcid == NULL) {
-		asc_result_add_hint (cres,
-				     cpt,
-				     "internal-error",
-				     "msg", "No global ID could be found for component when processing screenshot images.",
-				     NULL);
+		asc_result_add_hint (
+		    cres,
+		    cpt,
+		    "internal-error",
+		    "msg",
+		    "No global ID could be found for component when processing screenshot images.",
+		    NULL);
 		return FALSE;
 	}
 
@@ -464,12 +483,15 @@ asc_process_screenshot_images_lang (AscResult *cres,
 		max_img_size_str = g_format_size (max_size_bytes);
 		img_size_str = g_format_size (img_data_len);
 		asc_result_add_hint (cres,
-					cpt,
-					"screenshot-image-too-big",
-					"fname", orig_img_url,
-					"max_size", max_img_size_str,
-					"size", img_size_str,
-					NULL);
+				     cpt,
+				     "screenshot-image-too-big",
+				     "fname",
+				     orig_img_url,
+				     "max_size",
+				     max_img_size_str,
+				     "size",
+				     img_size_str,
+				     NULL);
 		return FALSE;
 	}
 
@@ -494,32 +516,41 @@ asc_process_screenshot_images_lang (AscResult *cres,
 		/* save the source screenshot as PNG image */
 		src_image = asc_image_new_from_data (img_data,
 						     img_data_len,
-						     0, /* destination size */
+						     0,	    /* destination size */
 						     FALSE, /* compressed */
 						     ASC_IMAGE_LOAD_FLAG_NONE,
 						     &error);
 		if (error != NULL) {
-			g_autofree gchar *msg = g_strdup_printf ("Could not load source screenshot for storing: %s", error->message);
+			g_autofree gchar *msg = g_strdup_printf (
+			    "Could not load source screenshot for storing: %s",
+			    error->message);
 			asc_result_add_hint (cres,
 					     cpt,
 					     "screenshot-save-error",
-					     "url", orig_img_url,
-					     "error", msg,
+					     "url",
+					     orig_img_url,
+					     "error",
+					     msg,
 					     NULL);
 			return FALSE;
 		}
 
 		if (!asc_image_save_filename (src_image,
-						src_img_path,
-						0, 0,
-						ASC_IMAGE_SAVE_FLAG_OPTIMIZE,
-						&error)) {
-			g_autofree gchar *msg = g_strdup_printf ("Can not store source screenshot: %s", error->message);
+					      src_img_path,
+					      0,
+					      0,
+					      ASC_IMAGE_SAVE_FLAG_OPTIMIZE,
+					      &error)) {
+			g_autofree gchar *msg = g_strdup_printf (
+			    "Can not store source screenshot: %s",
+			    error->message);
 			asc_result_add_hint (cres,
 					     cpt,
 					     "screenshot-save-error",
-					     "url", orig_img_url,
-					     "error", msg,
+					     "url",
+					     orig_img_url,
+					     "error",
+					     msg,
 					     NULL);
 			return FALSE;
 		}
@@ -530,8 +561,10 @@ asc_process_screenshot_images_lang (AscResult *cres,
 
 		source_scr_width = asc_image_get_width (src_image);
 		source_scr_height = asc_image_get_height (src_image);
+		source_scr_scale = as_image_get_scale (orig_img);
 		as_image_set_width (simg, source_scr_width);
 		as_image_set_height (simg, source_scr_height);
+		as_image_set_scale (simg, source_scr_scale);
 
 		/* if we should not create a screenshots store, delete the just-downloaded file and set
 		 * the original upstream URL as source.
@@ -567,19 +600,27 @@ asc_process_screenshot_images_lang (AscResult *cres,
 		if (target_height > source_scr_height)
 			continue;
 
+		/* NOTE: we ignore higher scaling factors for thumbnailing for now */
+		if (source_scr_scale > 1)
+			continue;
+
 		thumb = asc_image_new_from_data (img_data,
 						 img_data_len,
-						 0, /* destination size */
+						 0,	/* destination size */
 						 FALSE, /* compressed */
 						 ASC_IMAGE_LOAD_FLAG_NONE,
 						 &error);
 		if (error != NULL) {
-			g_autofree gchar *msg = g_strdup_printf ("Could not load source screenshot for thumbnailing: %s", error->message);
+			g_autofree gchar *msg = g_strdup_printf (
+			    "Could not load source screenshot for thumbnailing: %s",
+			    error->message);
 			asc_result_add_hint (cres,
 					     cpt,
 					     "screenshot-save-error",
-					     "url", orig_img_url,
-					     "error", msg,
+					     "url",
+					     orig_img_url,
+					     "error",
+					     msg,
 					     NULL);
 			g_error_free (g_steal_pointer (&error));
 			continue;
@@ -592,31 +633,38 @@ asc_process_screenshot_images_lang (AscResult *cres,
 
 		/* create thumbnail storage path and URL component*/
 		if (g_strcmp0 (locale, "C") == 0)
-			thumb_img_name = g_strdup_printf ("image-%i_%ix%i.png",
-							  scr_no,
-							  asc_image_get_width (thumb),
-							  asc_image_get_height (thumb));
-		else
-			thumb_img_name = g_strdup_printf ("image-%i_%ix%i_%s.png",
+			thumb_img_name = g_strdup_printf ("image-%i_%ix%i@%i.png",
 							  scr_no,
 							  asc_image_get_width (thumb),
 							  asc_image_get_height (thumb),
+							  1);
+		else
+			thumb_img_name = g_strdup_printf ("image-%i_%ix%i@%i_%s.png",
+							  scr_no,
+							  asc_image_get_width (thumb),
+							  asc_image_get_height (thumb),
+							  1,
 							  locale);
 		thumb_img_path = g_build_filename (scr_export_dir, thumb_img_name, NULL);
 		thumb_img_url = g_build_filename (scr_base_url, thumb_img_name, NULL);
 
 		/* store the thumbnail image on disk */
 		if (!asc_image_save_filename (thumb,
-						thumb_img_path,
-						0, 0,
-						ASC_IMAGE_SAVE_FLAG_OPTIMIZE,
-						&error)) {
-			g_autofree gchar *msg = g_strdup_printf ("Can not store thumbnail image: %s", error->message);
+					      thumb_img_path,
+					      0,
+					      0,
+					      ASC_IMAGE_SAVE_FLAG_OPTIMIZE,
+					      &error)) {
+			g_autofree gchar *msg = g_strdup_printf (
+			    "Can not store thumbnail image: %s",
+			    error->message);
 			asc_result_add_hint (cres,
 					     cpt,
 					     "screenshot-save-error",
-					     "url", orig_img_url,
-					     "error", msg,
+					     "url",
+					     orig_img_url,
+					     "error",
+					     msg,
 					     NULL);
 			g_error_free (g_steal_pointer (&error));
 			continue;
@@ -638,8 +686,9 @@ asc_process_screenshot_images_lang (AscResult *cres,
 		asc_result_add_hint (cres,
 				     cpt,
 				     "screenshot-no-thumbnails",
-				     "url", orig_img_url,
-				      NULL);
+				     "url",
+				     orig_img_url,
+				     NULL);
 
 	return TRUE;
 }
@@ -649,7 +698,7 @@ asc_process_screenshot_images_lang (AscResult *cres,
  *
  * Returns: (transfer full): The processed screenshot.
  */
-static AsScreenshot*
+static AsScreenshot *
 asc_process_screenshot_images (AscResult *cres,
 			       AsComponent *cpt,
 			       AsScreenshot *scr,
@@ -664,9 +713,9 @@ asc_process_screenshot_images (AscResult *cres,
 	GHashTableIter ht_iter;
 	gpointer ht_key, ht_value;
 	g_autoptr(GHashTable) ht_lang_img = g_hash_table_new_full (g_str_hash,
-								   g_str_equal,
-								   g_free,
-								   g_object_unref);
+								    g_str_equal,
+								    g_free,
+								    g_object_unref);
 
 	imgs = as_screenshot_get_images_all (scr);
 	if (imgs->len == 0) {
@@ -684,13 +733,11 @@ asc_process_screenshot_images (AscResult *cres,
 					     g_object_ref (img));
 	}
 
-
-
 	/* just take the first image if we couldn't find a source */
 	if (g_hash_table_size (ht_lang_img) == 0)
 		g_hash_table_insert (ht_lang_img,
-					g_strdup ("C"),
-					g_object_ref (AS_IMAGE (g_ptr_array_index (imgs, 0))));
+				     g_strdup ("C"),
+				     g_object_ref (AS_IMAGE (g_ptr_array_index (imgs, 0))));
 
 	/* drop metainfo images */
 	as_screenshot_clear_images (scr);
@@ -739,17 +786,19 @@ asc_process_screenshots (AscResult *cres,
 	if (media_export_root == NULL)
 		store_screenshots = FALSE;
 
-	screenshots = as_component_get_screenshots (cpt);
+	screenshots = as_component_get_screenshots_all (cpt);
 	if (screenshots->len == 0)
 		return;
 
 	gcid = asc_result_gcid_for_component (cres, cpt);
 	if (gcid == NULL) {
-		asc_result_add_hint (cres,
-				     cpt,
-				     "internal-error",
-				     "msg", "No global ID could be found for component when processing screenshots.",
-				     NULL);
+		asc_result_add_hint (
+		    cres,
+		    cpt,
+		    "internal-error",
+		    "msg",
+		    "No global ID could be found for component when processing screenshots.",
+		    NULL);
 		return;
 	}
 
@@ -763,7 +812,11 @@ asc_process_screenshots (AscResult *cres,
 	if (media_url_prefix == NULL)
 		scr_base_url = g_build_path (G_DIR_SEPARATOR_S, gcid, "screenshots", NULL);
 	else
-		scr_base_url = g_build_path (G_DIR_SEPARATOR_S, media_url_prefix, gcid, "screenshots", NULL);
+		scr_base_url = g_build_path (G_DIR_SEPARATOR_S,
+					     media_url_prefix,
+					     gcid,
+					     "screenshots",
+					     NULL);
 
 	/* process screenshots */
 	valid_scrs = g_ptr_array_new_with_free_func (g_object_unref);
